@@ -1,3 +1,4 @@
+import paramiko
 from powerprotect.ppdm import Ppdm
 from powerprotect.credential import Credential
 from powerprotect import exceptions
@@ -15,6 +16,35 @@ accpeted_types = ["DATADOMAINMANAGEMENTCENTER", "SMISPROVIDER", "DDSYSTEM",
 
 
 class AssetSource(Ppdm):
+    """ Class to define an Asset Source object
+
+    Asset source object creation require: name, server, password
+    At object creation this library will go gather any information about
+    this object, if it exists on the PPDM server.
+
+    There are five interactive methods intended for user interaction:
+        create_assetsource
+        get_assetsource (run after creation and after all modification
+                         operations automatically)
+        update_assetsource
+        delete_assetsource
+        remove_all_assets_from_policies
+
+    Attributes:
+        name (str): Display name of the asset source (unique)
+        id (str): Unique ID of the asset source (unique)
+        type (str): Asset source type, only from list of acepted types
+        body (dict): Return payload from PPDM server.
+        target_body (dict): Body to be used for updates to the object
+        exists (bool): Used to detirmine if the object exists on the PPDM
+                       server
+        check_mode (bool): If true pretend to modify object on the PPDM server
+        msg (str): Return message of previous operations
+        failure (bool): If true, the previous operation failed
+        fail_msg (str): Detailed message from previous failure
+        fail_response (dict): Payload returned from PPDM after a failure
+        assets (list): List of all assets that are part of this asset source
+    """
     def __init__(self, **kwargs):
         try:
             self.name = kwargs['name']
@@ -25,11 +55,12 @@ class AssetSource(Ppdm):
             self.exists = False
             self.changed = False
             self.check_mode = kwargs.get('check_mode', False)
+            self.discovery = {}
             self.msg = ""
             self.failure = False
             self.fail_msg = ""
             self.fail_response = {}
-            self.assests = {}
+            self.assets = []
             super().__init__(**kwargs)
             if 'token' not in kwargs:
                 super().login()
@@ -38,13 +69,39 @@ class AssetSource(Ppdm):
             assetsource_logger.error(f"Missing required field: {e}")
             raise exceptions.PpdmException(f"Missing required field: {e}")
 
-    def create_assetsource(self, **kwargs):
-        address = kwargs['address']
-        credential_name = kwargs['credential_name']
+    def create_assetsource(self, credential_id=None, credential_name=None,
+                           **kwargs):
+        """ Method to create asset source if not present
+
+        This method will create an asset source if not already present. After
+        this method runs the get_assetsource method will execute updating the
+        objects attributes.
+
+        Check_Mode: True
+
+        Args:
+            address (str): FQDN or IP of asset source
+            port (int): Port to communicate with asset source
+            asset_type (str): PPDM Asset Source Type
+            credential_name (str): Unique name of cred to use for auth
+            tanzu (bool, default: False): For k8s tanzu clusters specify True
+            vcenter_name (str): If Tanzu k8s, specify unique vcenter name. This
+                                must already be an asset source in PPDM
+            enable_vsphere_integration (bool): For vcenter asset source type,
+                                               set to True in order to enable.
+
+        Returns:
+            None
+        """
+        try:
+            address = kwargs['address']
+            port = kwargs['port']
+            asset_type = (kwargs['asset_type']).upper()
+        except KeyError as e:
+            raise exceptions.PpdmException(f"Missing required argument: {e}")
         tanzu = kwargs.get('tanzu', False)
         vcenter_name = kwargs.get('vcenter_name', '')
-        port = kwargs['port']
-        asset_type = (kwargs['asset_type']).upper()
+        vcenter_id = kwargs.get('vcenter_id', '')
         enable_vsphere_integration = kwargs.get('enable_vsphere_integration',
                                                 False)
         if not self.exists:
@@ -52,8 +109,10 @@ class AssetSource(Ppdm):
                 return_body = self.__create_assetsource(
                     address=address,
                     credential_name=credential_name,
+                    credential_id=credential_id,
                     tanzu=tanzu,
                     vcenter_name=vcenter_name,
+                    vcenter_id=vcenter_id,
                     port=port,
                     asset_type=asset_type,
                     enable_vsphere_integration=enable_vsphere_integration)
@@ -74,18 +133,49 @@ class AssetSource(Ppdm):
         self.get_assetsource()
 
     def get_assetsource(self):
+        """ Method to gather asset source information if present
+
+        This method can be thought of as a sync between PPDM and an asset
+        source object. As such it is run at object creation time, and after
+        each of the other four methods. This method updates attributes only
+
+        Args:
+            None
+
+        Returns:
+            None
+        """
         assetsource = self.__get_assetsource_by_name()
         if bool(assetsource.response) is not False:
             self.exists = True
             self.id = assetsource.response['id']
             self.type = assetsource.response['type']
             self.assets = self.__get_all_assets()
+            self.__get_asset_source_discovery()
         else:
             self.exists = False
             self.id = ""
+            self.assets = []
+            self.discovery = {}
         self.body = assetsource.response
 
     def update_assetsource(self):
+        """ Method to update asset source if present
+
+        This method uses the target_body attribute to update the asset source
+        in PPDM. If there is no asset sourcein PPDM  or target_body
+        attribute, nothing will happen. After this method completes, the
+        get_assetsource method will run and the target_body attribute will
+        clear itself.
+
+        Check_Mode: True
+
+        Args:
+            None
+
+        Returns:
+            None
+        """
         if (self.exists and
                 self.target_body and
                 helpers._body_match(self.body, self.target_body) is False):
@@ -107,6 +197,20 @@ class AssetSource(Ppdm):
         self.get_assetsource()
 
     def delete_assetsource(self):
+        """ Method to destroy asset source if present
+
+        This method will delete the asset source from PPDM. If the asset source
+        does not exist nothing will happen. After this method completes the
+        get_assetsource method will run to update this objects attributes.
+
+        Check_Mode: True
+
+        Args:
+            None
+
+        Returns:
+            None
+        """
         if self.exists:
             if not self.check_mode:
                 return_body = self.__delete_assetsource()
@@ -125,24 +229,98 @@ class AssetSource(Ppdm):
         self.get_assetsource()
 
     def remove_all_assets_from_policies(self):
+        """ Method to remove an asset sources assets from any protection
+            policies
+
+        This method finds any assets from the objects asset source that belong
+        to a protection policy. Typically used before removing an asset
+        source. After this method completes the get_assetsource method will
+        run to update the objects attributes.
+
+        Check_Mode: False
+
+        Args:
+            None
+
+        Returns:
+            None
+        """
         assetsource_logger.debug("Method: remove_all_assets_from_policies")
         if self.exists:
             for asset in self.assets:
                 if asset['protectionPolicyId']:
                     body = [asset['id']]
-                    response = super()._rest_post("/protection-policies"
-                                                  f"/{asset['protectionPolicyId']}"
-                                                  "/asset-unassignments", body)
+                    url = ("/protection-policies/"
+                           f"{asset['protectionPolicyId']}"
+                           "/asset-unassignments")
+                    response = super()._rest_post(url, body)
                     if response.ok is False:
-                        assetsource_logger.error("Unable to remove asset:"
-                                                 f"{asset['name']} from policy:"
-                                                 f"{asset['protectionPolicy']['name']}")
+                        assetsource_logger.error(f"""Unable to remove asset:
+                                                 {asset['name']} from policy:
+                                                 {asset['protectionPolicy']
+                                                 ['name']}""")
                     if response.ok:
-                        assetsource_logger.debug("Successfully removed asset:"
-                                                 f"{asset['name']} from policy:"
-                                                 f"{asset['protectionPolicy']['name']}")
+                        assetsource_logger.debug(f"""Successfully removed
+                                                 asset: {asset['name']} from
+                                                 policy: {asset['protection'
+                                                 'Policy']['name']}""")
             self.get_assetsource()
 
+    def add_root_cert(self, **kwargs):
+        assetsource_logger.debug("Method: add_root_cert")
+        try:
+            username = kwargs['ssh_username']
+            password = kwargs['ssh_password']
+            base64_cert = kwargs['base64_cert']
+        except KeyError as e:
+            assetsource_logger.error(f"Missing required field: {e}")
+            raise exceptions.PpdmException(f"Missing required field: {e}")
+        command = ("/usr/local/brs/bin/ppdmtool -importcert -alias "
+                   f"{self.name} -file /home/admin/cert.pem -type BASE64")
+        ssh = paramiko.SSHClient()
+        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+        try:
+            ssh.connect(self.server, username=username, password=password)
+        except paramiko.ssh_exception.AuthenticationException as authError:
+            raise exceptions.PpdmException("Invalid auth credentials: "
+                                           f"{authError}")
+        ftp = ssh.open_sftp()
+        file = ftp.file('cert.pem', 'w', -1)
+        file.write(base64_cert)
+        file.flush()
+        ftp.close()
+        stdin, stdout, stderr = ssh.exec_command(command, timeout=30)
+        error = stderr.read()
+        error = error.decode("utf-8")
+        output = stdout.read()
+        output = output.decode("utf-8")
+        ssh.close()
+        if not ("Certificate was added to keystore" in error or
+                "already exists" in output):
+            assetsource_logger.error("The certificate was not able to be "
+                                     "added to the trust store.")
+            assetsource_logger.error(f"stderr: {error}")
+            assetsource_logger.error(f"stdout: {output}")
+            return False
+        assetsource_logger.debug("Certificate was successfully added")
+        return True
+
+    def __get_asset_source_discovery(self):
+        assetsource_logger.debug("Method: __get_asset_source_discovery")
+        response = super()._rest_get("/discoveries?filter=start%20eq%20%22%2F"
+                                     f"inventory-sources%2F{self.id}%22")
+        if not response.json()['content']:
+            assetsource_logger.error("Unable to find discovery schedule")
+        self.discovery = response.json()['content'][0]
+
+    def __update_asset_source_discovery(self):
+        assetsource_logger.debug("Method: __update_asset_source_discovery")
+        response = super()._rest_put(f"/discoveries/{self.discovery['id']}",
+                                     self.discovery)
+        if not response.ok:
+            assetsource_logger.error("Unable to update discovery id: "
+                                     f"{self.discovery['id']}")
+
     def __get_all_assets(self):
         assetsource_logger.debug("Method: __get_all_assets")
         if self.type == "KUBERNETES":
@@ -170,7 +348,7 @@ class AssetSource(Ppdm):
         if response.ok:
             if not response.json()['content']:
                 err_msg = f"Assetsource not found: {self.name}"
-                assetsource_logger.info(err_msg)
+                assetsource_logger.debug(err_msg)
                 return_body.success = True
                 return_body.status_code = response.status_code
                 return_body.response = {}
@@ -224,23 +402,33 @@ class AssetSource(Ppdm):
         return_body = helpers.ReturnBody()
         certificate = self.__get_host_certificate(kwargs['address'],
                                                   kwargs['port'])
-        self.__accept_host_certificate(certificate.response)
-        credential = Credential(name=kwargs['credential_name'],
-                                server=self.server,
-                                token=self._token)
+        if not certificate.response['state'] == 'ACCEPTED':
+            assetsource_logger.debug("Cert not accepted. Accepting now")
+            self.__accept_host_certificate(certificate.response)
+        credential_id = kwargs.get('credential_id', '')
+        if not credential_id:
+            credential = Credential(name=kwargs['credential_name'],
+                                    server=self.server,
+                                    token=self._token)
+            credential_id = credential.id
         body = {}
         body['address'] = kwargs['address']
         body['name'] = self.name
         body['port'] = kwargs['port']
         body['type'] = kwargs['asset_type']
-        body.update({'credentials': {'id': credential.id}})
+        vcenter_id = kwargs.get('vcenter_id', '')
+        body.update({'credentials': {'id': credential_id}})
         if kwargs['asset_type'] == 'KUBERNETES' and kwargs['tanzu']:
-            vcenter = self.__get_assetsource_by_name(name=kwargs
-                                                     ['vcenter_name'])
-            if vcenter.success:
-                body.update({'details':
-                             {'k8s':
-                              {'vCenterId': vcenter.response['id']}}})
+            if not vcenter_id:
+                vcenter = self.__get_assetsource_by_name(name=kwargs
+                                                         ['vcenter_name'])
+                vcenter_id = vcenter.response['id']
+            if not vcenter_id:
+                raise exceptions.PpdmException("Invalid or missing: "
+                                               "vcenter_name or vcenter_id")
+            body.update({'details':
+                         {'k8s':
+                          {'vCenterId': vcenter_id}}})
         if kwargs['asset_type'] == 'VCENTER':
             body.update({'details':
                          {'vCenter':
@@ -250,6 +438,10 @@ class AssetSource(Ppdm):
         if response.ok:
             msg = f"Assetsource id \"{self.name}\" " \
                    "successfully created"
+            self.id = response.json()['id']
+            self.__get_asset_source_discovery()
+            self.discovery['schedule']['enabled'] = True
+            self.__update_asset_source_discovery()
             return_body.success = True
         else:
             msg = f"Assetsource id \"{self.name}\" " \
@@ -264,8 +456,8 @@ class AssetSource(Ppdm):
     def __get_host_certificate(self, address, port):
         assetsource_logger.debug("Method: __get_host_certificate")
         return_body = helpers.ReturnBody()
-        response = super()._rest_get(f"/certificates?host={address}&port={port}"
-                                     "&type=Host")
+        response = super()._rest_get(f"/certificates?host={address}&"
+                                     f"port={port}&type=Root")
         if response.ok is False:
             return_body.success = False
             return_body.fail_msg = response.json()
