@@ -134,9 +134,11 @@ class ProtectionRule(Ppdm):
                     'conditions': [{
                         'assetAttributeName': 'userTags',
                         'operator': 'EQUALS',
-                        'assetAttributeValue': label
+                        'assetAttributeValues': [label]
                     }],
                     'connditionConnector': 'AND',
+                    'logicalOperator': 'AND',
+                    'category': 'COMPOUND',
                     'inventorySourceType': inventory_type,
                     'priority': kwargs.get('priority', 1),
                     'tenant': {
